///-------------------------------------------------------------------------------------------------
/// 
/// \file TactileMenu.cpp
/// \brief Definitions for class TactileMenu
/// \author Tobias Baumann (TH Nuernberg).
/// \version 1.0.
/// \date Mon Feb 10 17:11:34 2022.
/// \copyright <a href="https://choosealicense.com/licenses/agpl-3.0/">GNU Affero General Public License v3.0</a>.
/// 
///-------------------------------------------------------------------------------------------------

//includes
#include "TactileMenu.h"

#include <string>

namespace OpenViBE {
namespace Plugins {
namespace Tactilebci {

//TactileMenu Memberfunctions
TactileMenu::TactileMenu(char* label0, char* label1, char* label2, char* label3, char* label4, char* label5)
{
    m_LabelText[0] = label0;
    m_LabelText[1] = label1;
    m_LabelText[2] = label2;
    m_LabelText[3] = label3;
    m_LabelText[4] = label4;
    m_LabelText[5] = label5;
	
	m_MainInterface = nullptr;
    
    for(int i = 0; i < 6; i++)
    {
        m_SubMenu[i] = this;
        m_SubRoutine[i] = nullptr;
		m_Label[i] = nullptr;
    }
}

void TactileMenu::call_SubRoutine(int i)
{
    if(m_SubRoutine[i] != nullptr)
	{
        (*m_SubRoutine[i])();
	}
}

char* TactileMenu::get_LabelText(int i)
{
    return(m_LabelText[i]);
}

TactileMenu* TactileMenu::get_SubMenu(int i)
{
    return(m_SubMenu[i]);
}

void TactileMenu::set_SubRoutine(int i, void(*subroutine)(void))
{
    m_SubRoutine[i] = subroutine;
}

void TactileMenu::set_SubMenu(int i, TactileMenu* submenu)
{
    m_SubMenu[i] = submenu;
}

void TactileMenu::reloadui(void)
{
	for(int i = 0; i < 6; i++)
	{
		gtk_label_set_text(m_Label[i], m_LabelText[i]);
	}
}

void TactileMenu::init_Labels(GtkBuilder* maininterface)
{
	m_MainInterface = maininterface;
	
	m_Label[0] = GTK_LABEL(gtk_builder_get_object(maininterface, "label-1"));
	m_Label[1] = GTK_LABEL(gtk_builder_get_object(maininterface, "label-2"));
	m_Label[2] = GTK_LABEL(gtk_builder_get_object(maininterface, "label-3"));
	m_Label[3] = GTK_LABEL(gtk_builder_get_object(maininterface, "label-4"));
	m_Label[4] = GTK_LABEL(gtk_builder_get_object(maininterface, "label-5"));
	m_Label[5] = GTK_LABEL(gtk_builder_get_object(maininterface, "label-6"));
}


//Subroutine Functions
void dummysubroutine(void)
{	
}

}  // namespace Tactilebci
}  // namespace Plugins
}  // namespace OpenViBE